import React, { useState, useRef, useEffect } from 'react';
import styled from 'styled-components';

const ChatContainer = styled.div`
  max-width: 800px;
  margin: 0 auto;
  padding: 20px;
`;

const ChatBox = styled.div`
  border: 1px solid #ddd;
  border-radius: 8px;
  height: 400px;
  overflow-y: auto;
  padding: 20px;
  margin-bottom: 20px;
  background: #fff;
`;

const MessageBubble = styled.div`
  margin: 10px 0;
  padding: 10px 15px;
  border-radius: 15px;
  max-width: 70%;
  ${props => props.isUser ? `
    background: #007bff;
    color: white;
    margin-left: auto;
  ` : `
    background: #f1f1f1;
    margin-right: auto;
  `}
`;

const InputContainer = styled.div`
  display: flex;
  gap: 10px;
  align-items: flex-start;
`;

const Textarea = styled.textarea`
  width: 100%;
  font-size: 1em;
  padding: 8px;
  border: 1px solid #ccc;
  border-radius: 5px;
  box-sizing: border-box;
  resize: vertical;
  min-height: 38px;
  max-height: 200px;
  overflow: auto;
  transition: border 0.2s;
  &:focus {
    border: 1.5px solid #3498db;
    outline: none;
  }
`;

const Button = styled.button`
  padding: 10px 20px;
  background: #007bff;
  color: white;
  border: none;
  border-radius: 4px;
  cursor: pointer;
  &:hover {
    background: #0056b3;
  }
  &:disabled {
    background: #cccccc;
  }
`;

const FileUploadArea = styled.div`
  border: 2px dashed #ddd;
  padding: 20px;
  text-align: center;
  margin-bottom: 20px;
  border-radius: 4px;
  background: ${props => props.isDragging ? '#e9ecef' : 'transparent'};
`;

const FileList = styled.div`
  margin-top: 10px;
  
  .file-item {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 5px;
    background: #f8f9fa;
    margin: 5px 0;
    border-radius: 4px;
  }
`;

const Spinner = styled.div`
  display: ${props => props.visible ? 'block' : 'none'};
  text-align: center;
  margin: 10px 0;
  &:after {
    content: '';
    display: inline-block;
    width: 20px;
    height: 20px;
    border: 2px solid #f3f3f3;
    border-top: 2px solid #3498db;
    border-radius: 50%;
    animation: spin 1s linear infinite;
  }
  @keyframes spin {
    0% { transform: rotate(0deg); }
    100% { transform: rotate(360deg); }
  }
`;

const ChatInterface = () => {
  const [messages, setMessages] = useState([]);
  const [input, setInput] = useState('');
  const [files, setFiles] = useState([]);
  const [isDragging, setIsDragging] = useState(false);
  const [isLoading, setIsLoading] = useState(false);
  const fileInputRef = useRef();
  const chatBoxRef = useRef();

  const handleDrag = (e) => {
    e.preventDefault();
    e.stopPropagation();
  };

  const handleDragIn = (e) => {
    e.preventDefault();
    e.stopPropagation();
    setIsDragging(true);
  };

  const handleDragOut = (e) => {
    e.preventDefault();
    e.stopPropagation();
    setIsDragging(false);
  };

  const handleDrop = (e) => {
    e.preventDefault();
    e.stopPropagation();
    setIsDragging(false);
    
    const droppedFiles = Array.from(e.dataTransfer.files);
    handleFiles(droppedFiles);
  };

  const handleFileInput = (e) => {
    const selectedFiles = Array.from(e.target.files);
    handleFiles(selectedFiles);
  };

  const handleFiles = async (newFiles) => {
    const formData = new FormData();
    newFiles.forEach(file => {
      formData.append('files[]', file);
    });

    try {
      setIsLoading(true);
      const nodeId = window.drupalSettings.openai_search.current_node_id;
      const response = await fetch(`/api/openai-search/upload/${nodeId}`, {
        method: 'POST',
        body: formData,
        credentials: 'same-origin', // This is important for the session cookie
      });

      let result;
      const contentType = response.headers.get('content-type');
      if (contentType && contentType.includes('application/json')) {
        result = await response.json();
      } else {
        const text = await response.text();
        throw new Error('Server returned non-JSON response: ' + text.substring(0, 100));
      }
      
      if (response.ok) {
        setFiles(prevFiles => [...prevFiles, ...newFiles]);
        setMessages(prev => [...prev, {
          text: `Files uploaded successfully: ${newFiles.map(f => f.name).join(', ')}`,
          isUser: false
        }]);
      } else {
        throw new Error(result.error || 'Upload failed');
      }
    } catch (error) {
      console.error('Upload failed:', error);
      setMessages(prev => [...prev, {
        text: `File upload failed: ${error.message}`,
        isUser: false
      }]);
    } finally {
      setIsLoading(false);
      refreshAttachments();
    }
  };

  const handleSubmit = async () => {
    if (!input.trim() || isLoading) return;

    const searchQuery = input.trim().replace(/ /g, '+');
    setMessages(prev => [...prev, { text: input, isUser: true }]);
    setInput('');
    setIsLoading(true);

    try {
      const { node_type, ai_url, restful_url, session_id } = window.drupalSettings.openai_search;
      const url = node_type === "private_gpts" 
        ? `${restful_url}/${ai_url}/${searchQuery}?UNIQUE_SESSION_COOKIE_NAME=${session_id}`
        : `${restful_url}/${searchQuery}?UNIQUE_SESSION_COOKIE_NAME=${session_id}`;

      const response = await fetch(url, {
        method: 'GET',
        headers: {
          'Accept': 'application/json',
        },
      });

      const data = await response.json();
      if (data) {
        setMessages(prev => [...prev, { 
          text: data.response || data.documents || 'No response received', 
          isUser: false 
        }]);
      }
    } catch (error) {
      console.error('Chat request failed:', error);
      setMessages(prev => [...prev, { 
        text: 'Error: Could not get response from AI', 
        isUser: false 
      }]);
    } finally {
      setIsLoading(false);
    }
  };

  const removeFile = (index) => {
    setFiles(prev => prev.filter((_, i) => i !== index));
  };

  const handleRemoveAttachment = async (fid, index) => {
    try {
      setIsLoading(true);
      const nodeId = window.drupalSettings.openai_search.current_node_id;
      const response = await fetch(`/api/openai-search/attachments/${nodeId}/remove`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        credentials: 'same-origin',
        body: JSON.stringify({ fid }),
      });
      const data = await response.json();
      if (!response.ok) throw new Error(data.error || 'Failed to remove file');
      setFiles(prev => prev.filter((file, i) => i !== index));
      setMessages(prev => [...prev, { text: 'File removed successfully.', isUser: false }]);
    } catch (err) {
      setMessages(prev => [...prev, { text: `Error removing file: ${err.message}`, isUser: false }]);
    } finally {
      setIsLoading(false);
      refreshAttachments();
    }
  };

  // Helper to render HTML in messages
  const renderMessage = (msg) => {
    if (!msg) return null;
    // Convert encoded and literal <br/> to real line breaks
    let html = msg.replace(/(&lt;br\/?&gt;|<br\/?&gt;)/g, '<br />');
    return <span dangerouslySetInnerHTML={{ __html: html }} />;
  };

  // Helper to refresh file attachments after upload/remove
  const refreshAttachments = async () => {
    await fetchAttachments();
  };

  // Fetch file attachments on mount
  const fetchAttachments = async () => {
    try {
      const nodeId = window.drupalSettings.openai_search.current_node_id;
      const response = await fetch(`/api/openai-search/attachments/${nodeId}`, {
        credentials: 'same-origin',
      });
      if (!response.ok) throw new Error('Failed to fetch attachments');
      const data = await response.json();
      if (data.attachments && Array.isArray(data.attachments)) {
        // Convert to File-like objects for display, but mark as readonly
        setFiles(data.attachments.map(att => ({
          ...att,
          readonly: true,
        })));
      }
    } catch (err) {
      // Optionally show an error in the chat
      setMessages(prev => [...prev, { text: `Could not load attachments: ${err.message}`, isUser: false }]);
    }
  };

  // Scroll to bottom when messages change
  React.useEffect(() => {
    if (chatBoxRef.current) {
      chatBoxRef.current.scrollTop = chatBoxRef.current.scrollHeight;
    }
  }, [messages]);

  return (
    <ChatContainer>
      <FileUploadArea
        isDragging={isDragging}
        onDragEnter={handleDragIn}
        onDragLeave={handleDragOut}
        onDragOver={handleDrag}
        onDrop={handleDrop}
        onClick={() => fileInputRef.current.click()}
      >
        <input
          type="file"
          ref={fileInputRef}
          onChange={handleFileInput}
          style={{ display: 'none' }}
          multiple
        />
        <p>Drop files here or click to upload</p>
        <FileList>
          {files.map((file, index) => (
            <div key={index} className="file-item">
              <span>
                {file.url ? (
                  <a href={file.url} target="_blank" rel="noopener noreferrer">{file.name}</a>
                ) : (
                  file.name
                )}
                {file.size && (
                  <span style={{ marginLeft: 8, color: '#888', fontSize: '0.9em' }}>({(file.size/1024).toFixed(1)} KB)</span>
                )}
              </span>
              {!file.readonly ? (
                <Button onClick={e => { e.stopPropagation(); removeFile(index); }}>Remove</Button>
              ) : (
                <Button onClick={e => { e.stopPropagation(); handleRemoveAttachment(file.fid, index); }} disabled={isLoading}>Remove</Button>
              )}
            </div>
          ))}
        </FileList>
      </FileUploadArea>

      <ChatBox ref={chatBoxRef}>
        {messages.map((message, index) => (
          <MessageBubble key={index} isUser={message.isUser}>
            {renderMessage(message.text)}
          </MessageBubble>
        ))}
        <Spinner visible={isLoading} />
      </ChatBox>

      <InputContainer>
        <Textarea
          value={input}
          onChange={e => setInput(e.target.value)}
          onKeyDown={e => {
            if (e.key === 'Enter' && !e.shiftKey) {
              e.preventDefault();
              handleSubmit();
            }
          }}
          placeholder="Type your message..."
          disabled={isLoading}
          rows={1}
          style={{ minHeight: '38px', maxHeight: '200px', resize: 'vertical', overflow: 'auto' }}
        />
        <Button onClick={handleSubmit} disabled={isLoading || !input.trim()}>
          Send
        </Button>
      </InputContainer>
    </ChatContainer>
  );
};

export default ChatInterface;
